/*
 * Licensed Materials - Property of IBM
 * 5725-I43 (C) Copyright IBM Corp. 2014. All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
var jwt = require("jsonwebtoken");

var DEFAULT_SCOPE = "wl-default-scope";
var TOKEN_TYPE = "Bearer";

var responseCode = {
	UNAUTHORIZED : 401,
	FORBIDDEN : 403
};

var lastVersionSupportedStr = "1.0";
var lastVersionSupported = parseVersion(lastVersionSupportedStr);

var tokenError = {
	INVALID : {reason : "invalid_token", stat : responseCode.UNAUTHORIZED},
	MISSING: {reason : null, stat : responseCode.UNAUTHORIZED},
	INSUFFICIENT : {reason : "insufficient_scope", stat : responseCode.FORBIDDEN}
};

var tokenErrorMsg = {
	EXPIRED : "The token has expired",
	SCOPE : "The token scope is different than the expected scope",
	VERSION : formatStr("Unsupported token version. The last version supported is '{0}'", lastVersionSupportedStr),
	PARSE : "Unable to parse token data",
	FORMAT : "The token does not have the expected format"
};

module.exports = function(cert, requiredScope) {
	function validateHeader(authorizationHeader, callback) {
		var token = parseToken(authorizationHeader);
		validateToken(token, callback);
	}

	function validateToken(token, callback) {
		if (!token) {
			return callback(createErrorObj(null, tokenError.MISSING, requiredScope), null);
		}
		jwt.verify(token, cert, function(errorMsg, tokenData) {
			if (errorMsg) {
				return callback(createErrorObj(errorMsg, tokenError.INVALID, requiredScope), null);				
			}
			validateTokenData(tokenData, callback);

		});
	}

	function validateTokenData(tokenData, callback) {
	try {
		var parsedData = JSON.parse(tokenData);
		if (!isTokenStructureValid(parsedData)) {
			// Token data isn't as expected
			callback(createErrorObj(tokenErrorMsg.FORMAT, tokenError.INVALID, requiredScope), null);
//		} else if (!versionSupported(parsedData.wl_token_version)) {
//			// Version is not supported
//			callback(createErrorObj(tokenErrorMsg.VERSION, tokenError.INVALID, requiredScope, parsedData.scope), null);
		} else if (Date.now() > parsedData.expiration) {
			// Token is expired
			callback(createErrorObj(tokenErrorMsg.EXPIRED, tokenError.INVALID, requiredScope, parsedData.scope), null);
		} else if (requiredScope && parsedData.scope != requiredScope) {
			// The scope of token is different than the scope required
			callback(createErrorObj(tokenErrorMsg.SCOPE, tokenError.INSUFFICIENT, requiredScope), null);
		} else {
			// Token is valid
			callback(null, parsedData);
		}
	} catch (e) {
		callback(createErrorObj(tokenErrorMsg.PARSE + " " + e, tokenError.INVALID, requiredScope), null);
	}
}

	return {
		validate : validateToken,
		validateAuthorizationHeader : validateHeader,
		_validateTokenData : validateTokenData
	};
};

function createErrorObj(desc, errorData, requiredScope, tokenScope) {
	// If scope given upon initialization, it will be used.
	// If not, scope of token is used, and if that doesn't exist (badly structured token) - default scope
	var headerScope = requiredScope? requiredScope : tokenScope? tokenScope : DEFAULT_SCOPE;
	// The format of returned header should be as follows:
	// WWW-Authenticate: Bearer scope=<scope>[,error=<error_code>][,error_description=<description>]
	var header = null;
	if (errorData && errorData.reason && desc) {
		header = formatStr("{0} scope={1},error={2},error_description={3}",
				TOKEN_TYPE, headerScope, errorData.reason, desc);
	} else {
		// In case of missing token, 
		header = formatStr("{0} scope={1}", TOKEN_TYPE, headerScope);
	}

	return {
		err : errorData.reason,
		errMsg : desc,
		httpStatus : errorData.stat,
		wwwAuthenticateHeader : header
	};
}

function formatStr(string) {
	for (var i = 1; i < arguments.length; i++) {
		string = string.replace("{" + (i-1) + "}", arguments[i]);
	}
	return string;
}

function parseToken(header) {
	var tokenPrefix = TOKEN_TYPE + " ";
	// Check if has the correct prefix
	if (header && header.indexOf(tokenPrefix) === 0) {
		return header.substring(tokenPrefix.length);
	} else {
		return null;
	}
}

function parseVersion(version) {
    var pattern = /^(\d+)\.(\d+)$/;
    var match = pattern.exec(version);
    if (match) return {major : parseInt(match[1]), minor : parseInt(match[2])};
}


function versionSupported(version) {
	var tokenVersion = parseVersion(version);
	if (!tokenVersion) return false;
	if (tokenVersion.major > lastVersionSupported.major) return false;
	if (tokenVersion.major < lastVersionSupported.major) return true;
	return (tokenVersion.minor <= lastVersionSupported.minor);
}

function isTokenStructureValid(parsedData) {
	var expectedFields = ["scope", "expiration", "data"]; //"wl_token_version", 
	var keys = Object.keys(parsedData);
	for (var i = 0; i < expectedFields.length; i++ ) {
		if (keys.indexOf(expectedFields[i]) == -1) return false;
	}
	return true;
}